// PropBox.h
//      Definition of the CPropertyBox class
//      Written by Christopher Dix, 2000
#ifndef __PROPERTYBOX_H
#define __PROPERTYBOX_H

#if _MSC_VER > 1000
#pragma once
#endif // _MSC_VER > 1000

// CPropertyBox child control notification messages
#define PN_DOWNVAL                  WM_USER + 1000
#define PN_UPVAL                    WM_USER + 1001
#define PN_TOGGLELIST               WM_USER + 1002
#define PN_ENTERVAL                 WM_USER + 1003
#define PN_ESCVAL                   WM_USER + 1004

// CPropertyBox notification messages
#define PBN_PROPCHANGE              WM_USER + 1100
#define PBN_OWNERDROP               WM_USER + 1200

// Child window ids
#define PB_EDIT_ID                  1111
#define PB_VALBUTTON_ID             1222
#define PB_ODBUTTON_ID              1333

// Margin for value area of property box
#define kValueMargin                2

// Defaults for buttons
#define kDefButtonWidth             18
#define kDefOwnerDropButtonText     "..."

// Custom value used to make child edit read-only
#define kAllowEdit                  1000

// Flags for individual properties
typedef enum
{
    PROP_READONLY =     0x0001,    // Read only property
    PROP_EDIT =         0x0002,    // Edit control
    PROP_DROPLIST =     0x0004,    // Drop down list of values
    PROP_OWNERDROP =    0x0008,    // Drop down, owner handles
    PROP_GETCHANGE =    0x0010     // Notify of changes
} PROPFLAGS;

// CPropertyItem
//      Class representing a single property in a CPropertyBox
class CPropertyItem
{
public:
    CPropertyItem()
    {
        m_nFlags = PROP_EDIT | PROP_GETCHANGE;
        m_nValueIndex = -1;
        m_pLB = NULL;
    }
    
    virtual ~CPropertyItem() {;}
    
    CPropertyItem( const CPropertyItem& pi )
    {
        Set( pi );
    }

    CPropertyItem operator =( const CPropertyItem& pi )
    {
        CPropertyItem newPI( pi );
        return( newPI );
    }

    void Set( const CPropertyItem& pi )
    {
        m_strProperty = pi.m_strProperty;
        m_strValue = pi.m_strValue;
        m_nValueIndex = pi.m_nValueIndex;
        m_pLB = pi.m_pLB;
        m_nFlags = pi.m_nFlags;
    }

public:
    CString     m_strProperty;      // Property name
    CString     m_strValue;         // Property value
    short       m_nValueIndex;      // Value list index
    CListBox*   m_pLB;              // Drop down list box
    UINT        m_nFlags;           // Flags
};

// Property item notification display information
typedef struct tagNMPBDISPINFO : public NMHDR
{
    CPropertyItem*  pItem;          // Property item
    RECT            rectOwnerDrop;  // Suggested rect for drop down
} NMPBDISPINFO, FAR *LPNMPBDISPINFO; 

// CPropertyBox 
//      Generic property display window, derived from CListBox
class CPropertyBox : public CListBox
{
// Construction/Destruction
public:
    CPropertyBox();
    virtual ~CPropertyBox();

// Overrides
public:
    // ClassWizard generated virtual function overrides
    //{{AFX_VIRTUAL(CPropertyBox)
    public:
    virtual BOOL PreTranslateMessage(MSG* pMsg);
    protected:
    virtual BOOL PreCreateWindow(CREATESTRUCT& cs);
    virtual BOOL OnCommand(WPARAM wParam, LPARAM lParam);
    virtual void PreSubclassWindow();
    //}}AFX_VIRTUAL

// Operations
public:
    // Add/delete/select properties
    int AddProperty( CPropertyItem* pItem );
    int DeleteProperty( LPCTSTR lpszProperty );
    void RemoveAll();
    void SelectProperty( const int& nIndex );
    
    // Get property information
    CPropertyItem* GetProperty( const int& nIndex );
    CPropertyItem* GetProperty( LPCTSTR lpszProperty );
    
    // Get/set property values
    bool GetValue( LPCTSTR lpszProperty, CString& strValue );
    bool SetValue( LPCTSTR lpszProperty, LPCTSTR lpszValue );

// Access
public:
    // Get/SetTabWithinBox toggles TAB key handling
    inline bool GetTabWithinBox( void )
    { return( m_bTabWithinBox ); }
    
    inline void SetTabWithinBox( const bool& bTabWithinBox )
    { m_bTabWithinBox = bTabWithinBox; }
    
    // This font is used to calculate the height of a list item.  
    // It must be set before Create() is called or it defaults to
    // the font of the parent window.
    CFont*  m_pMeasureFont;

    // Generated message map functions
protected:
    //{{AFX_MSG(CPropertyBox)
    afx_msg void OnSelchange();
    afx_msg void MeasureItem(LPMEASUREITEMSTRUCT
                        lpMeasureItemStruct);
    afx_msg void DrawItem(LPDRAWITEMSTRUCT lpDrawItemStruct);
    afx_msg void OnSize(UINT nType, int cx, int cy);
    afx_msg UINT OnGetDlgCode();
    afx_msg void OnSetfocus();
    //}}AFX_MSG
    DECLARE_MESSAGE_MAP()

protected:
    // Floating child window members
    CRichEditCtrl*          m_pEdit;
    CButton*                m_pButton;
    CButton*                m_pODButton;
    CListBox*               m_pList;
    
    // Map of property names to values.
    CMapStringToPtr         m_mapPropToData;
    
    // Drawing measurement data.
    CString                 m_strLongestPropName;
    int                     m_nPropertyWidth;
    CRect                   m_rectValue;
    
    // Option flags
    bool                    m_bTabWithinBox;

protected:
    // Notifications
    bool SendNotification( UINT nCode, CPropertyItem*& pItem, 
        const RECT& rect );
    
    // Notify the application of a change to a property's value
    inline bool SendPropChange( CPropertyItem*& pItem )
    {
        return( SendNotification(PBN_PROPCHANGE, pItem, 
            CRect()));
    }
    
    // Notify the application of owner-handled drop down
    inline void SendOwnerDrop( CPropertyItem*& pItem, 
        const RECT& rect )
    {
        SendNotification( PBN_OWNERDROP, pItem, rect );
    }
    
    // Toggle the display of the value list for a property.
    void ToggleList( CPropertyItem* pItem );
    
    // Child window subclass procedure
    static LRESULT APIENTRY ChildSubclassProc( HWND hwnd, 
        UINT uMsg, WPARAM wParam, LPARAM lParam);
    
    // Hide the floating child windows
    void HideChildren( void );
};
#endif      // __PROPERTYBOX_H
